/*****************************************************************
   OtBbNfoCX for Cygwin ppcsim
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include "grlib.h"

#define TEXTHEIGHT 13
#define TEXTWIDTH   7

/* gdev configuration */
static int _width, _height;
static int doublebuf = 0;   /* 1: draw back then copy to front */

/****************************************************************/
static HWND    win;           /* main window    */
static HDC     gdev;          /* fore or back   */
static HDC     hdc;           /* foreground DC  */
static HDC     hdc_back;      /* background DC  */
static HBITMAP bmp_back;      /* background BMP */
static HFONT   hfont;
static HPEN    current_pen   = 0;
static HBRUSH  current_brush = 0;
/****************************************************************/

static LRESULT CALLBACK window_proc(HWND hWnd, UINT wMsg,
				    WPARAM wParam, LPARAM lParam)
{
  switch (wMsg){
  case WM_PAINT:
    BitBlt(hdc, 0, 0, _width, _height, hdc_back, 0, 0, SRCCOPY);
    return DefWindowProc(hWnd, wMsg, wParam, lParam);
    //    case WM_CLOSE:
    //	DestroyWindow(hWnd);
    //	PostQuitMessage(0);
    //	exit(0);
  default:
    return DefWindowProc(hWnd, wMsg, wParam, lParam);
  }
}

void Init_gdev(int width,int height, char *title)
{
  int w, h;
  RECT rect;
  WNDCLASS wclass;
  //DWORD wstyle = (0)? WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX
  //                    : WS_OVERLAPPEDWINDOW;
  DWORD wstyle =  WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX ;
  
  wclass.lpszClassName ="Display";
  wclass.lpszMenuName  = NULL;
  wclass.hInstance     = GetModuleHandle(NULL);
  wclass.lpfnWndProc   = window_proc;
  wclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wclass.hIcon         = NULL;
  wclass.hbrBackground = GetStockObject(BLACK_BRUSH);
  wclass.style         = 0;
  wclass.cbClsExtra    = 0;
  wclass.cbWndExtra    = 0;
  
  if (!RegisterClass(&wclass))  exit(1);
  
  _width = width; _height= height;
  SetRect(&rect, 0, 0, width, height);
  AdjustWindowRect(&rect, wstyle, 0);
  w = rect.right  - rect.left;
  h = rect.bottom - rect.top;
  
  win = CreateWindow(wclass.lpszClassName, title, wstyle,
		     CW_USEDEFAULT, CW_USEDEFAULT,
		     w, h, NULL, NULL, wclass.hInstance, NULL);
  ShowWindow(win, SW_SHOWDEFAULT);
  
  /* set as default */
  gdev = hdc = GetDC(win);
  
  /* set up background bitmap */
  hdc_back = CreateCompatibleDC(hdc);
  bmp_back = CreateCompatibleBitmap(hdc, width, height);
  SelectObject(hdc_back, bmp_back);
  
  /* font */
  hfont = CreateFont(TEXTHEIGHT, TEXTWIDTH, 0, 0, 
		     0, FALSE, FALSE, FALSE, DEFAULT_CHARSET,
		     OUT_DEFAULT_PRECIS,CLIP_TT_ALWAYS,PROOF_QUALITY,
		     FIXED_PITCH|FF_ROMAN, "TERMINAL");
  SelectObject(hdc, hfont);
  SelectObject(hdc_back, hfont);
  
  FlushConsoleInputBuffer(GetStdHandle(STD_INPUT_HANDLE));
}

void FlushBuffer(void)
{
  GdiFlush();
  if (doublebuf)
    BitBlt(hdc, 0, 0, _width, _height, hdc_back, 0, 0, SRCCOPY);
  else 
    BitBlt(hdc_back, 0, 0, _width, _height, hdc, 0, 0, SRCCOPY);
}

void Close_gdev(void)
{
  if (win) {
    DeleteDC(hdc);
    DeleteDC(hdc_back);
    DeleteObject(bmp_back);
    if (current_pen) DeleteObject(current_pen);
    if (current_brush) DeleteObject(current_brush);
    UnregisterClass("Display", GetModuleHandle(NULL));
    win = 0;
  }
}

static void (*mouseCB)(int button, int state, double x, double y) = NULL;

void SetMouseFunc(void (*mouse)(int, int, double, double)) { mouseCB = mouse; }

static void mouseEvent(MSG msg)
{
  static int lftdwn=0;
  POINT pt;
  
  GetCursorPos(&pt);
  ScreenToClient(win, &pt);
  switch (msg.message) {
  case WM_RBUTTONDOWN:
    mouseCB(RIGHT_BUTTON, MOUSE_DOWN, pt.x, pt.y);
    return;
  case WM_RBUTTONUP:
    mouseCB(RIGHT_BUTTON, MOUSE_UP, pt.x, pt.y);
    return;
  case WM_LBUTTONDOWN:
    mouseCB(LEFT_BUTTON, MOUSE_DOWN, pt.x, pt.y);
    lftdwn=1; return;
  case WM_LBUTTONUP:
    mouseCB(LEFT_BUTTON, MOUSE_UP, pt.x, pt.y);
    lftdwn=0; return;
  case WM_MOUSEMOVE:
    if (lftdwn == 1)
      mouseCB(LEFT_BUTTON, MOUSE_MOVE, pt.x, pt.y);
    return;
  default:
    return;
  }
}

static int last_key = -1;

int KbHit(void)
{
  MSG msg;
  while (PeekMessage(&msg,0,0,0,PM_REMOVE)) {
    TranslateMessage(&msg);
    if (mouseCB) {
      mouseEvent(msg);
      return 1;
    }
    if (msg.message == WM_CHAR) {
      last_key = msg.wParam;
      return 1;
    } else{
      DispatchMessage(&msg);
      return 1;
    }
  }
  return 0;
}

int GetKey(void)
{
  int k;
  
  while (last_key == -1)
    KbHit();
  k = last_key;
  last_key = -1;
  return k;
}

void Clear_gdev(void)
{
  HBRUSH new_brush;
  COLORREF bg_color ;
  
  bg_color=RGB(0,0,0) ;
  new_brush = CreateSolidBrush(bg_color);
  SelectObject(gdev, new_brush);
  if (current_brush) DeleteObject(current_brush);
  current_brush = new_brush;
  Rectangle(gdev, -10, -10, _width+10, _height+10);
}

void DrawPix(int x, int y, COLORREF pix)
{
  SetPixel(gdev, x, y, pix);
}
